"""Constants for the Panasonic Viera integration."""
DOMAIN = "panasonic_viera"

DEVICE_MANUFACTURER = "Panasonic"

CONF_ON_ACTION = "turn_on_action"
CONF_APP_ID = "app_id"
CONF_ENCRYPTION_KEY = "encryption_key"

DEFAULT_NAME = "Panasonic Viera TV"
DEFAULT_PORT = 55000

ATTR_REMOTE = "remote"

ERROR_NOT_CONNECTED = "not_connected"
ERROR_INVALID_PIN_CODE = "invalid_pin_code"

REASON_NOT_CONNECTED = "not_connected"
REASON_UNKNOWN = "unknown"
