"""Constants for the HVV Departure integration."""

DOMAIN = "hvv_departures"
DEFAULT_NAME = DOMAIN
MANUFACTURER = "HVV"
ATTRIBUTION = "Data provided by www.hvv.de"

CONF_STATION = "station"
CONF_REAL_TIME = "real_time"
CONF_FILTER = "filter"
